#ifndef TETRAKAIDECAHEDRA_H
#define TETRAKAIDECAHEDRA_H

#include "vmRandom.h" // for distorting uniform Tetrakaidecahedra
#include <math.h>
#include <cstdlib>
#include <stdio.h>
#include <fstream>
#include <iostream>
using namespace std;

// grain deformation class developed by Rongshan Qin at GIFT, POSTECH
// the paper is being published at ISIJ International

class Tetrakaidecahedra{
public:
	Tetrakaidecahedra(int numberOfGrainX, int numberOfGrainY, int numberOfGrainZ, double grainSize);
	~Tetrakaidecahedra();
	void calcVtxCoord();
	void getOneTetrakaideVtx(int & xpos, int & ypos, int & zpos);
	double getEdgeOfOneUniformTetra();
	double getSurfaceAreaOfOneUniformTetra();
	double getEdgeOfOneRandomTetra();
	double getSurfaceAreaOfOneRandomTetra();
	double getVolumeOfOneRandomTetra();
	void deformVtx();
	void ramdomizeVtx(double randomIntensity);
	void setPlainDeformMatrix(double s11);
	void setAxisymmetricTensionDeformMatrix(double s11);
	void setAxisymmetricCompressionDeformMatrix(double s33);
	void setSimpleShearDeformMatrix(double s13);
	void setDeformationMatrix(double s11, double s12, double s13, 
							  double s21, double s22, double s23, 
							  double s31, double s32, double s33);
	double getSurfaceAreaOfAllRandomTetra();
	double getEdgeOfAllRandomTetra();
	double getVolumeOfAllRandomTetra();
	void zoomAllRandomTetra(int caseNumber);
	void outputVertex(char* filename);
	int totalGrainNumber(){return ngt;};
	int totalVertexNumber(){return nvt;};
	void sortVolumeCategary();
	double getPrimaryEdgeOfAllRandomTetra();
	double getSecondaryEdgeOfAllRandomTetra();
	double getPrimaryEdgeOfOneRandomTetra();
	double getSecondaryEdgeOfOneRandomTetra();
private:
	vmRandom rdm;
	ofstream file;
	ofstream volfile;
	double distance(int point1, int point2); 
	double trangleArea(int point1, int point2, int point3);
	double quadArea(int point1, int point2, int point3, int point4);
	double hexagonArea(int point1, int point2, int point3, int point4, int point5, int point6);
	bool atSamePlane(int point1, int point2, int point3, int point4);
	double getNaturalEdgeLengthOfOneTetra();
	double getAnomalousEdgeLengthOfOneTetra();
	double getVolumeTriangularPyramid(int point1, int point2, int point3);
	void setOneTetrakaideVtx(int & xpos, int & ypos, int & zpos);
	void deformGrainCenter();
	void getGrainCentrePos(int & xpos, int & ypos, int & zpos);
	void zoomOneGrainVtxTowardCenter(int & xpos, int & ypos, int & zpos, double zoom);
	int nvt; // number of vertex
	int ngx,ngy,ngz,ngt;
	int xtg, ytg, ztg;  // xtg = ngx/2+2
	double gs, qgs, hgs;
	double xgc, ygc, zgc;
	double dbacurate;
	float maxdim;
	double *vcx;
	double *vcy;
	double *vcz;
	double *vtx;
	double *sij;
};

#endif // TETRAKAIDECAHEDRA_H
